<?php
/*
 * 簡易投票スクリプト
 * 
 * HISTORY
 * 
 * 2008-03-04 version 1.0.1 リリース (クッキー発行にバグあり)
 * 2008-03-04 version 1.0.0 リリース
 *
 * @package 金魚屋「車輪の再開発」シリーズ
 * @copyright (C) 2008 金魚屋見聞録
 * @author 金魚屋・Ｊ・龍角
 * @license http://www.kingyoya.org/
 * @version 1.0.1
 *
 */

/*
 * 日本語環境の設定
 * 不要な場合は削除
 */
mb_language('Japanese');
mb_internal_encoding('utf8');
mb_http_output('utf8');
ob_start('mb_output_handler');

require_once './Kingyoya/SimpleVote.php';

/**
 * データを出力するCSV ファイルのパス
 * @var string
 */
define('DATA_FILE', './data.csv');
/**
 * 同時実行制御用ロックファイル
 * @var string
 */
define('LOCK_FILE', './.lock');
/**
 * 連続投票回避クッキー名
 * @var string
 */
define('COOKIE_NAME', 'voted');
/**
 * 連続投票回避クッキーの有効期限
 * @var int
 */
define('COOKIE_EXPIRE', 24*60*60);

/**
 * エラーログを出力するファイル
 * 必要な場合は設定
 * @var string
 */
define('ERROR_LOG_FILE', './error_log.txt');

if (defined('ERROR_LOG_FILE')) {
	/*
	 * エラーログファイルが指定されている場合の出力設定
	 */
	ini_set('error_reporting', E_ALL);
	ini_set('display_errors', 0);
	ini_set('log_errors', 1);
	ini_set('error_log', ERROR_LOG_FILE);
}

function redirect() {
	header('HTTP/1.0 303 See Other');
	header('Location: '. $_SERVER['REQUEST_URI']);
	exit;
}

if (!defined('DATA_FILE')) {
	die("Data file is not set");
}
if (!is_file(DATA_FILE)) {
	die("Data file is not file: ".DATA_FILE);
}
if (!is_readable(DATA_FILE)) {
	die("Data file is not readable file: ".DATA_FILE);
}
if (!is_writable(DATA_FILE)) {
	die("Data file is not writable file: ".DATA_FILE);
}

$votor = new Kingyoya_SimpleVote(DATA_FILE);

/**
 * 簡易投票スクリプト
 */
if (strtolower($_SERVER['REQUEST_METHOD']) === 'post') {
	/*
	 * 投票時処理
	 */
	if (isset($_COOKIE[COOKIE_NAME])) {
		trigger_error('Already voted', E_USER_NOTICE);
		redirect();
	}

	if (!isset($_POST['vote'])) {
		trigger_error('Failed get vote', E_USER_WARNING);
		redirect();
	}
	$vote = (string) $_POST['vote'];
	if (!$vote) {
		trigger_error('Failed get int vote', E_USER_WARNING);
		redirect();
	}

	$lock = fopen(LOCK_FILE, 'w');
	if (!$lock) {
		trigger_error('Failed open lock file: '.LOCK_FILE, E_USER_WARNING);
		redirect();
	}
	$result = flock($lock, LOCK_EX);
	if (!$lock) {
		trigger_error('Failed lock: '.LOCK_FILE, E_USER_WARNING);
		redirect();
	}

	$result = $votor->vote((string) $vote);
	if (!$result) {
		trigger_error('Failed vote', E_USER_WARNING);
	}

	fclose($lock);

	if (defined('COOKIE_EXPIRE') && COOKIE_EXPIRE) {
		setcookie(COOKIE_NAME, 1, time()+COOKIE_EXPIRE);
	} else {
		setcookie(COOKIE_NAME, 1);
	}

	redirect();

} else {
	/*
	 * 通常時処理
	 */
	$result = $votor->load();
	if (!$result) {
		trigger_error('Failed load data', E_USER_WARNING);
	}
}

/*
 * ここからHTML 表示
 */
echo '<?xml version="1.0" encoding="UTF-8" ?>';
?>

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja" lang="ja">
	<head>
		<meta http-equiv="content-type" content="text/html; charset=UTF8" />
		<title>SIMPLE VOTE SAMPLE</title>
	</head>
	<body>
		<h1>Simple Vote Sample</h1>
		<form action="index.php" method="post">
			<ul>
				<li><label><input name="vote" type="radio" value="1" title="hoge" />HOGE</label></li>
				<li><label><input name="vote" type="radio" value="2" title="fuga" />FUGA</label></li>
				<li><label><input name="vote" type="radio" value="3" title="piyo" />PIYO</label></li>
			</ul>
			<p><input type="submit" name="submit" value="投票" /></p>
		</form>
		<h2>投票結果(割合)</h2>
		<p>HOGE: <?php echo $votor->rate('1'); ?>%</p>
		<p>FUGA: <?php echo $votor->rate('2'); ?>%</p>
		<p>PIYO: <?php echo $votor->rate('3'); ?>%</p>
		<h2>投票結果(カウント)</h2>
		<p>総数: <?php echo $votor->total(); ?></p>
		<p>HOGE: <?php echo $votor->count('1'); ?></p>
		<p>FUGA: <?php echo $votor->count('2'); ?></p>
		<p>PIYO: <?php echo $votor->count('3'); ?></p>
		<hr />
		<address>
			Copyright (C) 2008 <a href="/">金魚屋見聞録</a>
		</address>
	</body>
</html>
